<?php

/**
 * @file
 * Views integration for time_tracker.
 */

/**
 * Implements hook_views_data().
 */
function time_tracker_views_data() {
  $data['time_tracker_entry']['table']['group'] = t('Time Tracker');
  $data['time_tracker_entry']['table']['base'] = array(
    'field' => 'teid',
    'title' => t('Time Tracker'),
    'help' => t('Content generated from time tracker entries.'),
  );

  $entity_types = entity_get_info();
  foreach ($entity_types as $key => $type) {
    $bundles = $type['bundles'];
    foreach ($bundles as $bkey => $bundle) {
      $tracking = time_tracker_is_tracking_time($key, $bkey);
      if ($tracking) {
        $data['time_tracker_entry'][$type['entity keys']['id']] = array(
          'title' => $type['label'],
          'help' => t('Relate an entry to its %label entity.', array('%label' => $type['label'])),
          'relationship' => array(
            'handler' => 'time_tracker_views_handler_relationship',
            'base' => $type['base table'],
            'base field' => $type['entity keys']['id'],
            'label' => t('%label ID', array('%label' => $type['label'])),
          ),
        );
        $data[$key]['time_tracker_entry'] = array(
          'title' => t('Time Tracker Entry'),
          'help' => t('Relate an entity (%bundle_label) to its Time Tracker entries.', array(
            '%bundle_label' => $bundle['label'],
          )),
          'relationship' => array(
            'handler' => 'time_tracker_views_handler_relationship',
            // This is a reverse relationship, so we need to reverse the meaning
            // of remote and base.
            'remote' => $type['base table'],
            'remote field' => $type['entity keys']['id'],
            'base' => 'time_tracker_entry',
            'base field' => 'entity_id',
            'label' => t('Time Tracker Entry'),
            'extra' => array(
              array(
                'table' => 'time_tracker_entry_' . $key,
                'field' => 'entity_type',
                'value' => $key,
              ),
              array(
                'table' => 'time_tracker_entry_' . $key,
                'field' => 'entity_bundle',
                'value' => $bkey,
              ),
            ),
          ),
        );
      }
    }
  }

  $data['time_tracker_entry']['teid'] = array(
    'title' => t('ID'),
    'help' => t('Time Entry ID.'),
    'field' => array(
      'handler' => 'views_handler_field_numeric',
    ),
    'sort' => array(
      'handler' => 'views_handler_sort_numeric',
    ),
    'filter' => array(
      'handler' => 'views_handler_filter_numeric',
    ),
    'argument' => array(
      'handler' => 'views_handler_argument',
    ),
  );

  $data['time_tracker_entry']['timestamp'] = array(
    'title' => t('Timestamp'),
    'help' => t('Timestamp of the Entry.'),
    'field' => array(
      'handler' => 'views_handler_field_date',
    ),
    'sort' => array(
      'handler' => 'views_handler_sort_date',
    ),
    'filter' => array(
      'handler' => 'time_tracker_views_handler_filter_date',
      'is date' => TRUE,
    ),
    'argument' => array(
      'handler' => 'views_handler_argument',
    ),
  );

  $data['time_tracker_entry']['start'] = array(
    'title' => t('Start Time'),
    'help' => t('Start time of the Entry.'),
    'field' => array(
      'handler' => 'views_handler_field_date',
    ),
    'sort' => array(
      'handler' => 'views_handler_sort_date',
    ),
    'filter' => array(
      'handler' => 'time_tracker_views_handler_filter_date',
      'is date' => TRUE,
    ),
    'argument' => array(
      'handler' => 'views_handler_argument',
    ),
  );

  $data['time_tracker_entry']['end'] = array(
    'title' => t('End Time'),
    'help' => t('End time of the Entry.'),
    'field' => array(
      'handler' => 'views_handler_field_date',
    ),
    'sort' => array(
      'handler' => 'views_handler_sort_date',
    ),
    'filter' => array(
      'handler' => 'time_tracker_views_handler_filter_date',
      'is date' => TRUE,
    ),
    'argument' => array(
      'handler' => 'views_handler_argument',
    ),
  );

  $data['time_tracker_entry']['duration'] = array(
    'title' => t('Duration'),
    'help' => t('The Raw Time Entry Duration (without deductions).'),
    'field' => array(
      'handler' => 'time_tracker_views_handler_field_numeric',
      'float' => TRUE,
    ),
    'sort' => array(
      'handler' => 'views_handler_sort',
    ),
    'filter' => array(
      'handler' => 'views_handler_filter_numeric',
    ),
    'argument' => array(
      'handler' => 'views_handler_argument',
    ),
  );

  $data['time_tracker_entry']['note'] = array(
    'title' => t('Note'),
    'help' => t('Notes for node-based time entries'),
    'field' => array(
      'handler' => 'views_handler_field',
    ),
    'sort' => array(
      'handler' => 'views_handler_sort',
    ),
    'filter' => array(
      'handler' => 'views_handler_filter',
    ),
    'argument' => array(
      'handler' => 'views_handler_argument',
    ),
  );

  $data['time_tracker_entry']['deductions'] = array(
    'title' => t('Deductions'),
    'help' => t('Time Entry Deductions.'),
    'field' => array(
      'handler' => 'time_tracker_views_handler_field_numeric',
      'float' => TRUE,
    ),
    'sort' => array(
      'handler' => 'views_handler_sort',
    ),
    'filter' => array(
      'handler' => 'views_handler_filter_numeric',
    ),
    'argument' => array(
      'handler' => 'views_handler_argument',
    ),
  );

  $data['time_tracker_entry']['total_duration'] = array(
    'title' => t('Total Duration'),
    'help' => t('Time Entry Total Duration, minus the deductions'),
    'field' => array(
      'field' => 'time_tracker_total_duration',
      'handler' => 'time_tracker_views_handler_field_total_duration',
      'float' => TRUE,
    ),
    'sort' => array(
      'handler' => 'views_handler_sort',
    ),
    'filter' => array(
      'handler' => 'views_handler_filter_numeric',
    ),
    'argument' => array(
      'handler' => 'views_handler_argument',
    ),
  );

  $data['time_tracker_entry']['billable'] = array(
    'title' => t('Billable'),
    'help' => t('Is the Entry Billable?'),
    'field' => array(
      'handler' => 'views_handler_field_boolean',
    ),
    'sort' => array(
      'handler' => 'views_handler_sort',
    ),
    'filter' => array(
      'handler' => 'views_handler_filter_boolean_operator',
    ),
    'argument' => array(
      'handler' => 'views_handler_argument',
    ),
  );

  $data['time_tracker_entry']['billed'] = array(
    'title' => t('Billed'),
    'help' => t('Has the Entry been billed?'),
    'field' => array(
      'handler' => 'views_handler_field_boolean',
    ),
    'sort' => array(
      'handler' => 'views_handler_sort',
    ),
    'filter' => array(
      'handler' => 'views_handler_filter_boolean_operator',
    ),
    'argument' => array(
      'handler' => 'views_handler_argument',
    ),
  );

  $data['time_tracker_entry']['locked'] = array(
    'title' => t('Locked'),
    'help' => t('Is the Entry Locked?'),
    'field' => array(
      'handler' => 'views_handler_field_boolean',
    ),
    'sort' => array(
      'handler' => 'views_handler_sort',
    ),
    'filter' => array(
      'handler' => 'views_handler_filter_boolean_operator',
    ),
    'argument' => array(
      'handler' => 'views_handler_argument',
    ),
  );

  $data['time_tracker_entry']['uid'] = array(
    'title' => t('Author'),
    'help' => t('Relate an entry to the user who created it.'),
    'relationship' => array(
      'handler' => 'views_handler_relationship',
      'base' => 'users',
      'base field' => 'uid',
      'label' => t('User'),
    ),
    'argument' => array(
      'handler' => 'views_handler_argument',
    ),
  );

  $data['time_tracker_entry']['comment_id'] = array(
    'title' => t('Comment'),
    'help' => t('Relate an entry to a comment.'),
    'relationship' => array(
      'handler' => 'views_handler_relationship',
      'base' => 'comment',
      'base field' => 'cid',
      'label' => t('Comment'),
    ),
  );

  $data['time_tracker_entry']['activity'] = array(
    'title' => t('Activity'),
    'help' => t('Relate an entry to an Activity Name.'),
    'relationship' => array(
      'handler' => 'views_handler_relationship',
      'base' => 'time_tracker_activity',
      'base field' => 'taid',
      'label' => t('Activity'),
    ),
  );

  $data['time_tracker_activity']['table']['join']['time_tracker_entry'] = array(
    'left_field' => 'activity',
    'field' => 'taid',
  );
  $data['time_tracker_activity']['table']['group'] = t('Time Tracker');

  $data['time_tracker_activity']['name'] = array(
    'title' => t('Activity Name'),
    'help' => t('Activity Name.'),
    'field' => array(
      'handler' => 'views_handler_field',
    ),
    'sort' => array(
      'handler' => 'views_handler_sort_sort',
    ),
    'filter' => array(
      'handler' => 'views_handler_filter_string',
    ),
    'argument' => array(
      'handler' => 'views_handler_argument_string',
    ),
  );

  return $data;
}

/**
 * Implements hook_date_views_fields().
 *
 * This is required to allow timetracker timestamp as a date argument in views.
 */
function time_tracker_date_views_fields($field) {
  $values = array(
    'sql_type' => DATE_UNIX,
    'tz_handling' => 'site',
    'timezone_field' => '',
    'offset_field' => '',
    'related_fields' => array(),
    'granularity' => array('year', 'month', 'day', 'hour', 'minute', 'second'),
  );

  switch ($field) {
    case 'time_tracker_entry.timestamp':
    case 'time_tracker_entry.start':
    case 'time_tracker_entry.end':
      return $values;
  }
}

/**
 * Implements hook_views_bulk_operations_object_info().
 */
function time_tracker_views_bulk_operations_object_info() {
  $object_info = array(
    'time_tracker_entry' => array(
      'type' => 'time_tracker',
      'base_table' => 'time_tracker_entry',
      'load' => '_time_tracker_entry_load',
      'oid' => 'teid',
      'title' => 'title',
    ),
  );
  return $object_info;
}
